import sys
import traceback
import time
import threading
import six
import os
import inspect
import site

from six.moves import builtins

from rook.logger import logger

from rook.config import ImportServiceConfig


class ImportService(object):

    NAME = "Import"

    class Notification(object):
        def __init__(self, module_name, module_filename, include_externals, callback, removed):
            self.module_name = module_name
            self.module_filename = module_filename
            self.callback = callback
            self.removed = removed
            self.include_externals = include_externals

    def __init__(self, bdb_location_service):
        self._bdb_location_service = bdb_location_service

        self._modules = dict(sys.modules)
        self._post_import_notifications = {}
        self._lock = threading.RLock()

        self._thread = None
        self._quit = False

        self._old_import = None

        external_paths = [sys.exec_prefix]
        if hasattr(site, 'getsitepackages'):
            external_paths = external_paths + site.getsitepackages()

        self.external_paths = [os.path.normcase(external_path) for external_path in external_paths]

        if ImportServiceConfig.USE_IMPORT_HOOK:
            self._old_import = builtins.__import__
            builtins.__import__ = self._my_import
        else:
            self._thread = threading.Thread(target=self._query_thread,
                                            name=ImportServiceConfig.THREAD_NAME)
            self._thread.daemon = True
            self._thread.start()

    def close(self):
        if self._old_import:
            builtins.__import__ = self._old_import

        if self._thread:
            self._quit = True

            # If threading was monkey patched by gevent waiting on thread will likely throw an exception
            try:
                from gevent.monkey import is_module_patched
                if is_module_patched("threading"):
                    time.sleep(ImportServiceConfig.SYS_MODULES_QUERY_INTERVAL)
            except:
                pass

            self._thread.join()

    def register_post_import_notification(self, aug_id, name, filename, include_externals, callback, removed):
        # Normalize file path
        if filename:
            filename = os.path.normcase(os.path.normpath(filename))

        notification = self.Notification(name, filename, include_externals, callback, removed)

        with self._lock:
            # Attempt to satisfy notification using known modules
            for module_name, module_object in six.iteritems(self._modules):

                # If module is not valid, ignore
                if not self._is_valid_module(module_object):
                    continue

                # Get module details and check if it matches
                module_filename = self._get_module_path(module_object)
                if module_filename:
                    if self._does_module_match_notification(module_name, module_filename, notification, self.external_paths):
                        notification.callback(module_object)
                        return False

            # Register notification for future loads
            self._post_import_notifications[aug_id] = notification

        return True

    def remove_aug(self, aug_id):
        with self._lock:
            try:
                notification = self._post_import_notifications[aug_id]
            except KeyError:
                return

            notification.removed()
            del self._post_import_notifications[aug_id]

    def clear_augs(self):
        with self._lock:
            for aug_id in list(self._post_import_notifications.keys()):
                self.remove_aug(aug_id)

            self._post_import_notifications.clear()

    def _is_valid_module(self, module_object):
        return module_object and inspect.ismodule(module_object) and hasattr(module_object, '__file__')

    def _my_import(self, name, globals=None, locals=None, fromlist=(), level=0):
        # Call original function
        result = self._old_import(name, globals, locals, fromlist, level)

        self._evaluate_module_list()

        # Return original result
        return result

    def _query_thread(self):
        self._bdb_location_service.ignore_current_thread()

        while not self._quit:
            try:
                self._evaluate_module_list()
            except:
                if logger:
                    logger.exception("Error while evaluating module list")

            # time can be None if interpreter is in shutdown
            if not time:
                return
            time.sleep(ImportServiceConfig.SYS_MODULES_QUERY_INTERVAL)

    def _evaluate_module_list(self):
        try:
            # Nobody is waiting for notifications
            if not self._post_import_notifications:
                return

            # No new modules
            if len(self._modules) == len(sys.modules):
                return

            with self._lock:

                # Get a fresh list
                modules = dict(sys.modules)
                # list of modules that are in the middle of being loaded
                pending = []

                # For everybody not in the old list, check notifications
                for name, module in six.iteritems(modules):
                    if name not in self._modules and self._is_valid_module(module):
                        # module without __builtins__ is considered to being loaded
                        # but not executed
                        if not hasattr(module, '__builtins__'):
                            pending.append(name)
                            continue

                        self._notify_of_new_module(name, module)

                # retry pending modules later
                for module_name in pending:
                    del modules[module_name]

                # Update the "old" list
                self._modules = modules

        except:
            logger.exception("Exception in ImportService")

    def _notify_of_new_module(self, module_name, module_object):
        augs_to_remove = set()

        module_filename = self._get_module_path(module_object)
        if module_filename:
            for aug_id, notification in six.iteritems(self._post_import_notifications):
                if self._does_module_match_notification(module_name, module_filename, notification, self.external_paths):
                    augs_to_remove.add(aug_id)
                    try:
                        notification.callback(module_object)
                    except:
                        logger.exception("Error on module load callback")

        for aug_id in augs_to_remove:
            del self._post_import_notifications[aug_id]


    @staticmethod
    def _get_module_path(module):
        try:
            path = inspect.getsourcefile(module)
        except:
            return None

        if path:
            return os.path.normcase(os.path.abspath(path))

        return None


    @staticmethod
    def _does_module_match_notification(module_name, module_filename, notification, external_paths):
        if not notification.include_externals:
            for external_path in external_paths:
                if module_filename.startswith(external_path):
                    return False

        if notification.module_filename and ImportService.path_contains_path(module_filename, notification.module_filename) or \
                        notification.module_name and module_name == notification.module_name:
            return True
        else:
            return False

    @staticmethod
    def path_contains_path(full_path, partial_path):
        if full_path.endswith(partial_path):
            return len(full_path) == len(partial_path) or full_path[-len(partial_path)-1] in ('/', '\\')
        else:
            return False



class IfCondition(object):
    NAME = 'if'

    def __init__(self, configuration, factory):
        self.path = factory.get_path(configuration["path"])

    def evaluate(self, namespace, extracted):
        obj = self.path.read_from(namespace).obj

        return isinstance(obj, bool) and obj



import logging
import six
import traceback

from ..processor.namespaces.container_namespace import ContainerNamespace
from ..processor.namespaces.python_object_namespace import PythonObjectNamespace


class NamespaceSerializerHandler(logging.Handler):

    output = None

    try :
        STRING_TYPES = (str, unicode)
    except NameError:
        STRING_TYPES = (str, )

    def __init__(self):
        super(NamespaceSerializerHandler, self).__init__()

    def emit(self, record):
        if not self.output:
            return

        try:
            arguments = {}

            try:
                formatted_message = record.getMessage()
            except:
                formatted_message = ""

            # If message is a pure string, record it, else add to arguments dictionary
            if isinstance(record.msg, self.STRING_TYPES):
                raw_message = record.msg
            else:
                raw_message = ""
                arguments['msg'] = PythonObjectNamespace(record.msg)

            # Add exception to arguments dictionary
            if record.exc_info:
                arguments['exc_type'] = PythonObjectNamespace(record.exc_info[0])
                arguments['exc_instance'] = PythonObjectNamespace(record.exc_info[1])
                arguments['traceback'] = PythonObjectNamespace(record.exc_info[2])

            # Add additional arguments to arguments dictionary
            if record.args:
                arguments['args'] = PythonObjectNamespace(record.args)

            # Send record information to output
            self.output.send_log_message(record.levelno, record.created, record.pathname, record.lineno, raw_message, formatted_message, ContainerNamespace(arguments))
        except Exception:
            six.print_("Unexpected error when writing to log")
            traceback.print_exc()



class ToolException(Exception):

    def __init__(self, *args):
        super(ToolException, self).__init__(*args)

    def get_type(self):
        return self.__class__.__name__

    def get_message(self):
        try:
            return self.args[0]
        except IndexError:
            return ""

    def get_parameters(self):
        try:
            return self.args[1]
        except IndexError:
            return ()


class RookException(ToolException):
    pass


class RookCommunicationException(RookException):
    pass


class RookInputException(ToolException):
    pass


class WebHookProcessingException(Exception):
    def __init__(self, status_code, content):
        super(WebHookProcessingException, self).__init__("Problem sending webhook %s",
                                                         {
                                                             "status_code": status_code,
                                                             "content": content
                                                         })


class WebHookRetryLevelException(Exception):
    def __init__(self, retry_level, valid_levels):
        super(WebHookRetryLevelException, self).__init__("Invalid retry level for webhook %s",
                                                         {
                                                        "retry_level": retry_level,
                                                        "valid_levels": valid_levels}
                                                         )


class RookInvalidArithmeticPath(RookInputException):
    def __init__(self, configuration):
        super(RookInvalidArithmeticPath, self).__init__("Invalid arithmetic path configuration %s",
                                                        {'configuration': configuration})


class RookInvalidBasicPath(RookInputException):
    def __init__(self, configuration):
        super(RookInvalidBasicPath, self).__init__("Invalid basic path configuration %s",
                                                   {'configuration': configuration})


class RookAugInvalidKey(RookInputException):
    def __init__(self, key, configuration):
        super(RookAugInvalidKey, self).__init__("Failed to get key from configuration %s",
                                                {'key': key,
                                                 'configuration': configuration})


class RookObjectNameMissing(RookInputException):
    def __init__(self, configuration):
        super(RookObjectNameMissing, self).__init__("Failed to find object name %s", {'configuration': configuration})


class RookUnknownObject(RookInputException):
    def __init__(self, object_name):
        super(RookUnknownObject, self).__init__("Failed to find object %s", {'object_name': object_name})


class RookInvalidObjectConfiguration(RookInputException):
    def __init__(self, object_name, object_config):
        super(RookInvalidObjectConfiguration, self).__init__("Failed to build object %s",
                                                             {'object_name': object_name,
                                                              'object_config': object_config})


class RookSendToRookoutDisabledException(RookInputException):
    def __init__(self):
        super(RookSendToRookoutDisabledException, self).__init__()


class RookMonitorException(RookException):
    pass


class RookHashFailedException(RookMonitorException):
    def __init__(self, module_name):
        super(RookHashFailedException, self).__init__("Failed to calculate hash %s", {'module': module_name})


class RookHashMismatchException(RookMonitorException):
    def __init__(self, filepath, expected, calculated):
        super(RookHashMismatchException, self).__init__("File hashes do not match! %s",
                                                        {'filepath': filepath,
                                                         'expected': expected,
                                                         'calculated': calculated})


class RookBdbFailedException(RookMonitorException):
    def __init__(self, result):
        super(RookBdbFailedException, self).__init__("Failed to set breakpoint %s", {'result': result})


class RookBdbCodeNotFound(RookMonitorException):
    def __init__(self, alternatives, filename, module_filename, line):
        super(RookBdbCodeNotFound, self).__init__("Failed to find code object! Alternative lines: %s",
                                                  {'alternatives': alternatives,
                                                   'filename': filename,
                                                   'module_filename': module_filename,
                                                   'line': line})


class RookBdbSetBreakpointFailed(RookMonitorException):
    def __init__(self):
        super(RookBdbSetBreakpointFailed, self).__init__("Failed to set breakpoint! %s")


class RookAttributeNotFound(RookMonitorException):
    def __init__(self, attribute):
        super(RookAttributeNotFound, self).__init__("Failed to get attribute %s", {'attribute': attribute})


class RookKeyNotFound(RookMonitorException):
    def __init__(self, key):
        super(RookKeyNotFound, self).__init__("Failed to get key %s", {'key': key})


class RookMethodNotFound(RookMonitorException):
    def __init__(self, namespace_type, method):
        super(RookMethodNotFound, self).__init__("Namespace method not found %s",
                                                 {'namespace': namespace_type.__name__,
                                                  'method': method})


class RookWriteAttributeNotSupported(RookMonitorException):
    def __init__(self, namespace_type, attribute):
        super(RookWriteAttributeNotSupported, self).__init__("Namespace does not support write %S",
                                                             {'namespace': namespace_type.__name__,
                                                              'attribute': attribute})


class RookOperationReadOnly(RookMonitorException):
    def __init__(self, operation_type):
        super(RookOperationReadOnly, self).__init__("Operation does not support write %s",
                                                    {'operation': operation_type.__name__})


class RookRuleRateLimited(RookException):
    def __init__(self):
        super(RookRuleRateLimited, self).__init__("Rule was disabled due to rate-limiting")


class RookNoHttpServiceRegistered(RookException):
    def __init__(self):
        super(RookNoHttpServiceRegistered, self).__init__("No http service registered")


class RookUnsupportedLocation(RookException):
    def __init__(self, location):
        super(RookUnsupportedLocation, self).__init__("Unsupported aug location was specified",
                                                      {'location': location})


class RookInterfaceException(RookException):
    def __init__(self, error_string):
        super(RookInterfaceException, self).__init__(error_string)


class RookVersionNotSupported(RookException):
    def __init__(self, error_string):
        super(RookVersionNotSupported, self).__init__(error_string)


class RookMissingToken(RookException):
    def __init__(self):
        super(RookMissingToken, self).__init__('Rookout token not supplied')


class RookInvalidToken(RookException):
    def __init__(self):
        super(RookInvalidToken, self).__init__('Rookout token was rejected')


class RookServiceMissing(RookException):
    def __init__(self, service):
        super(RookServiceMissing, self).__init__('Rookout service is missing',
                                                 {'service': service})


class RookInvalidOptions(RookException):
    def __init__(self, description):
        super(RookInvalidOptions, self).__init__(description)




from rook.processor.namespaces.container_namespace import ContainerNamespace
from rook.processor.namespaces.python_utils_namespace import PythonUtilsNamespace
from rook.processor.namespaces.stack_namespace import StackNamespace


class ActionRunProcessor(object):

    NAME = 'script'

    def __init__(self, configuration, processor_factory):
        self.processor = processor_factory.get_processor(configuration['operations'])

        if 'post_operations' in configuration:
            self.post_processor = processor_factory.get_processor(configuration['post_operations'])
        else:
            self.post_processor = None

    def execute(self, aug_id, frame, extracted_namespace, output):
        if not extracted_namespace:
            extracted_namespace = ContainerNamespace({})

        store = ContainerNamespace({})

        namespace = ContainerNamespace({
            'frame': frame,
            'stack': StackNamespace(frame),
            'extracted': extracted_namespace,
            'store': store,
            'temp': ContainerNamespace({}),
            'python': PythonUtilsNamespace(),
            'utils': PythonUtilsNamespace()
        })

        self.processor.process(namespace)
        output.send_user_message(aug_id, store)

        if self.post_processor:
            output.flush_messages()
            self.post_processor.process(namespace)



"""This package implements real-time data collection capabilities on top of the python environment.

In order to use Rookout data collection service, the following components are needed:
- The Rook module package (this package).
- The Rookout Agent.
- The Rookout Web Service.

For further information, checkout rookout.com.

This package can be used in two ways:
- Importing using "from rook import auto_start" will start the Rook using the default configuration.
- Importing "from rook import Rook" allows to manually configure and start the Rook.
"""

from .interface import start, flush



"""This is the external interface to the Rook package."""
import sys
import os
import traceback

import six

_rook = None
_debug = False
_silence_errors = True


def start(token=None,
          tags=None,
          host=None,
          port=None,
          debug=None,
          silence_errors=None,
          log_file=None,
          log_level=None,
          log_to_stderr=None,
          **kwargs):
    global _rook, _debug, _silence_errors

    if _rook is not None:
        return

    if isinstance(debug, bool):
        _debug = debug
    elif os.environ.get('ROOKOUT_DEBUG') == '1':
        _debug = True

    if isinstance(silence_errors, bool):
        _silence_errors = silence_errors

    if not isinstance(log_to_stderr, bool) and os.environ.get('ROOKOUT_LOG_TO_STDERR') == '1':
        log_to_stderr = True

    if tags is None and isinstance(os.environ.get('ROOKOUT_ROOK_TAGS'), str):
        raw_tags = os.environ.get('ROOKOUT_ROOK_TAGS')

        # copied from Namespace serializer so to not import it
        def normalize_string(obj):
            if six.PY2:
                if isinstance(obj, str):
                    return unicode(obj, errors="replace")
                else:
                    return unicode(obj)
            else:
                return str(obj)

        tags = [normalize_string(tag.strip()) for tag in raw_tags.split(';') if tag]

    log_file = log_file or os.environ.get('ROOKOUT_LOG_FILE')
    log_level = log_level or os.environ.get('ROOKOUT_LOG_LEVEL')
    host = host or os.environ.get('ROOKOUT_AGENT_HOST')
    port = port or os.environ.get('ROOKOUT_AGENT_PORT')
    token = token or os.environ.get('ROOKOUT_TOKEN')

    try:
        from rook.exceptions.tool_exceptions import RookMissingToken, RookInvalidToken, RookVersionNotSupported, \
            RookCommunicationException, RookInvalidOptions
        try:
            from rook.config import LoggingConfiguration

            if log_file is not None:
                if not isinstance(log_file, str):
                    raise RookInvalidOptions('Rook log file should be a String')
                LoggingConfiguration.FILE_NAME = log_file

            if log_level is not None:
                if not isinstance(log_level, str):
                    raise RookInvalidOptions('Rook log level should be a String')
                LoggingConfiguration.LOG_LEVEL = log_level

            if log_to_stderr is not None:
                LoggingConfiguration.LOG_TO_STDERR = log_to_stderr

            if _debug:
                LoggingConfiguration.LOG_LEVEL = 'DEBUG'
                LoggingConfiguration.LOG_TO_STDERR = True

            if isinstance(tags, list):
                for tag in tags:
                    if not isinstance(tag, str):
                        raise RookInvalidOptions('Rook tags should be array of strings')
            else:
                if tags:
                    raise RookInvalidOptions('Rook tags should be array of strings')

            if not host and not token:
                raise RookMissingToken()
            else:
                if token:
                    if not isinstance(token, str):
                        raise RookInvalidOptions('Rook token should be a String')

            import rook.singleton
            _rook = rook.singleton.singleton_obj

            return _rook.connect(token, host, port, tags)
        except (RookMissingToken, RookInvalidToken, RookVersionNotSupported) as e:
            six.print_("[Rookout] Failed to connect to the agent:", e, file=sys.stderr)
            raise
        except RookCommunicationException:
            six.print_("[Rookout] Failed to connect to the agent - will continue attempting in the background", file=sys.stderr)
            raise
        except ImportError as e:
            six.print_("[Rookout] Failed to import dependencies:", e, file=sys.stderr)
            raise
        except Exception as e:
            six.print_("[Rookout] Failed initialization:", e, file=sys.stderr)
            raise
    except Exception:
        if not _silence_errors:
            raise

        if _debug:
            traceback.print_exc()


def flush():
    global _rook
    _rook.flush()



import logging
import os
import sys
import six

from .handlers import RookoutStreamHandler, RookoutFileHandler
from .namespace_serializer_handler import NamespaceSerializerHandler


class LoggerFactory(object):
    def __init__(self, name, log_level, file_name, stderr, log_to_remote, propagate_logs):

        # Get the logger we'll use
        self.main_logger = logging.getLogger(name)
        self.main_logger.propagate = propagate_logs
        self.main_logger.setLevel(log_level)

        # Add flush functionality to logger
        self.main_logger.flush = self.flush

        # Remove old handlers if they exist, to prevent duplicates which cause multiple log entries
        handlers_dupe_list = list(self.main_logger.handlers)
        for log_handler in handlers_dupe_list:
            self.main_logger.removeHandler(log_handler)

        # Configure formatting
        self.formatting = r'%(asctime)s %(process)d:%(threadName)s- %(module)s:%(funcName)s@%(lineno)d - %(levelname)s - %(message)s %(rookout_extra)s'
        formatter = logging.Formatter(self.formatting)

        # Configure STD-ERR logging
        if stderr:
            console_stream = RookoutStreamHandler(sys.stderr)
            console_stream.setFormatter(formatter)
            self.main_logger.addHandler(console_stream)

        # GRPC logging
        self.main_logger.addHandler(NamespaceSerializerHandler())

        # Dwrite loggs to json
        if log_to_remote:
            from .json_handler import JsonHandler
            self.main_logger.addHandler(JsonHandler())

        # Configure File logging
        if file_name:
            if os.path.isabs(file_name):
                abs_log_path = file_name
            else:
                if "darwin" in sys.platform:
                    abs_log_path = os.path.join(os.getenv("HOME", "."), file_name)
                elif sys.platform == "win32":
                    abs_log_path = os.path.join(os.getenv("USERPROFILE", "."), file_name)
                else:
                    abs_log_path = os.path.join("/var/log", file_name)

            try:
                # Create directory if does not exist
                dirname = os.path.dirname(abs_log_path)
                if not os.path.isdir(dirname):
                    os.makedirs(dirname)

                # file handler
                file_handler = RookoutFileHandler(abs_log_path, maxBytes=10 * 1024 * 1024, backupCount=4)
                file_handler.setFormatter(formatter)
                self.main_logger.addHandler(file_handler)
            except Exception:
                self.main_logger.exception("Failed to open log file: %s", abs_log_path)

    def flush(self):
        for handler in self.main_logger.handlers:
            handler.flush()

    def get_logger(self):
        """

        :rtype: logging.Logger
        """
        return self.main_logger



import sys
import platform
import inspect
import importlib
import six

from rook.logger import logger
from rook.processor.namespaces.frame_namespace import FrameNamespace
from ..exceptions import RookBdbFailedException


class BdbLocationService(object):
    NAME = "Bdb"

    def __init__(self):
        self._bdb = self._get_bdb().Bdb()
        self._bdb.user_line = self.user_line

        self._breakpoints = dict()
        self._positions = dict()

        self._bdb.set_trace()

    def ignore_current_thread(self):
        self._bdb.ignore_current_thread()

    def add_breakpoint_aug(self, module, lineno, aug):
        filepath = inspect.getsourcefile(module)
        logger.info("Setting breakpoint at %s:%s", filepath, lineno)

        canonic = self._bdb.canonic(filepath)
        pos = (canonic, lineno)

        if pos in self._breakpoints:
            self._breakpoints[pos].append(aug)
        else:
            result = self._bdb.set_break(module, canonic, lineno)
            if result:
                raise RookBdbFailedException(result)
            else:
                self._breakpoints[pos] = [aug]

        self._positions[aug.aug_id] = pos
        aug.set_active()

    def remove_aug(self, aug_id):
        # Get current augs in position
        try:
            pos = self._positions[aug_id]
        except KeyError:
            return

        current_augs = self._breakpoints[pos]

        # Divide augs into delete and keep
        augs_to_delete = [aug for aug in current_augs if aug.aug_id == aug_id]
        augs_to_keep = [aug for aug in current_augs if aug.aug_id != aug_id]

        # Update list
        if augs_to_keep:
            self._breakpoints[pos] = augs_to_keep
        else:
            del self._breakpoints[pos]
            self._bdb.clear_break(pos[0], pos[1])

        # Update status
        for aug in augs_to_delete:
            aug.set_removed()

        # Remove from pos list
        del self._positions[aug_id]

    def clear_augs(self):
        # Attempt to clean nicely
        for aug_id in list(self._positions.keys()):
            self.remove_aug(aug_id)

        # Clean hard any left overs
        self._breakpoints = dict()
        self._positions = dict()
        self._bdb.clear_all_breaks()

    def close(self):
        self.clear_augs()
        self._bdb.close()

    def user_line(self, frame):
        try:
            filename = self._bdb.canonic(frame.f_code.co_filename)

            # Some Bdb implementations report line numbers before the break
            frame_namespace = FrameNamespace(frame)
            frame_namespace._lineno = self._bdb.fixed_line_number(frame)

            augs = self._breakpoints.get((filename, frame_namespace._lineno))

            if augs:
                for aug in augs:
                    aug.execute(frame_namespace, dict())
            else:
                logger.error("Aug not found! %s@%d", str(filename), frame_namespace._lineno)
        except:
            logger.exception("Error while processing breakpoint")

    def get_canonical_path(self, module):
        return self._bdb.canonic(inspect.getsourcefile(module))

    @staticmethod
    def _get_bdb():
        from rook.config import InstrumentationConfig
        if InstrumentationConfig.ENGINE == "auto":
            if 'CPython' == platform.python_implementation() and sys.platform in ('darwin', 'linux2', 'linux'):
                if (six.PY2 and sys.version_info > (2, 7, 0)) or (six.PY3 and sys.version_info > (3, 6, 0)):
                    from . import google_bdb as bdb
                else:
                    from . import py_bdb as bdb
            else:
                from . import py_bdb as bdb
        else:
            bdb = importlib.import_module(".." + InstrumentationConfig.ENGINE, __name__)

        return bdb



import six

from .dumped_object_namespace import DumpedObjectNamespace


class UserObjectNamespace(DumpedObjectNamespace):

    def __init__(self, type, attributes):
        super(UserObjectNamespace, self).__init__(type, u'UserObject', attributes)

    def to_dict(self):
        return {
            u'@namespace': self.__class__.__name__,
            u'@common_type': self.common_type,
            u'@original_type': self.type,
            u'@attributes': self.get_attributes_dict()
            }

    def to_simple_dict(self):
        return self.get_attributes_simple_dict()



from .namespace import Namespace


class ErrorNamespace(Namespace):

    def __init__(self, message, parameters, exception_namespace, traceback_namespace):
        super(ErrorNamespace, self).__init__()
        self.message = message
        self.parameters = parameters
        self.exc = exception_namespace
        self.traceback = traceback_namespace

    def to_dict(self):
        return {
            u'@namespace': self.__class__.__name__,
            u'@common_type': Namespace.__name__,
            u'@value': {
                u'message': self.message.obj,
                u'parameters': self.parameters.to_dict(),
                u'exc': self.exc.to_dict()
            }}

    def to_simple_dict(self):
        return {u'message': self.message.obj, u'parameters': self.parameters.to_simple_dict(), u'exc': self.exc.to_simple_dict()}



#!/usr/bin/env python
import platform
import sys
from glob import glob
import json

try:
    # Use setuptools if available, for install_requires (among other things).
    import setuptools
    from setuptools import setup, Extension
except ImportError:
    setuptools = None
    from distutils.core import setup, Extension

# Classic setup.py

kwargs = {}

# Version configuration
with open('rook/rookout-config.json', 'r') as f:
    config = json.load(f)
version = config['VersionConfiguration']['VERSION']

# Readme
with open('README.rst') as f:
    kwargs['long_description'] = f.read()

if 'CPython' == platform.python_implementation():
    ext_modules = []

    if sys.platform in ('darwin', 'linux2', 'linux') and (
            (sys.version_info[0] == 2 and sys.version_info > (2, 7, 0)) or
            (sys.version_info[0] == 3 and sys.version_info > (3, 6, 0))):
        extra_compile_args = [
            '-std=c++0x',
            '-g0',
            '-O3',
            '-Wno-deprecated-register'
        ]

        if sys.platform == 'darwin':
            extra_compile_args.append("-mmacosx-version-min=" + platform.mac_ver()[0])

        ext_modules.append(Extension(
            'rook.services.cdbg_native',
            sources=glob('rook/services/exts/cloud_debug_python/*.cc'),
            extra_compile_args=extra_compile_args))


    kwargs['ext_modules'] = ext_modules

if setuptools is not None:
    # If setuptools is not available, you're on your own for dependencies.
    install_requires = [
        "six >= 1.11",
        "grpcio >= 1.14.1",
        "protobuf >= 3.5.0, <= 4.0.0",
        "pyparsing"
    ]

    kwargs['install_requires'] = install_requires

setup(
    name="rook",
    version=version,
    packages=setuptools.find_packages(where='.', exclude=['contrib', 'docs', '*test*']),
    include_package_data=True,
    author="Rookout",
    author_email="liran@rookout.com",
    url="http://rookout.com/",
    description="Rook is a Python package for on the fly debugging and data extraction for application in production",
    license="https://get.rookout.com/SDK_LICENSE.pdf",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',

        ],
    zip_safe=False,
    **kwargs
)
